@@ -43,6 +43,27 @@ def test_construct_parameters(catalog_mock):
     assert search_parameters["sortby"] == mock_search_parameters["sortby"]
 
 
+def test_construct_parameters_fc_multiple_non_overlapping_features(catalog_mock):
+    with open(
+        Path(__file__).resolve().parent / "mock_data/search_footprints.geojson"
+    ) as json_file:
+        fc = json.load(json_file)
+
+    search_parameters = catalog_mock.construct_parameters(
+        geometry=fc,
+        start_date="2020-01-01",
+        end_date="2020-08-10",
+        sensors=["sentinel2"],
+        limit=10,
+        max_cloudcover=15,
+        sortby="acquisitionDate",
+        ascending=True,
+    )
+    assert isinstance(search_parameters, dict)
+    assert search_parameters["datetime"] == "2020-01-01T00:00:00Z/2020-08-10T00:00:00Z"
+    assert search_parameters["intersects"]["type"] == "MultiPolygon"
+
+
 def test_construct_parameters_unsopported_sensor_raises(catalog_mock):
     with pytest.raises(ValueError):
         catalog_mock.construct_parameters(
@@ -65,16 +86,15 @@ def test_search(catalog_mock):
         search_results = catalog_mock.search(mock_search_parameters)
 
     assert isinstance(search_results, gpd.GeoDataFrame)
-    assert search_results.shape == (1, 10)
+    assert search_results.shape == (1, 9)
 
 
 @pytest.mark.live
 def test_search_live(catalog_live):
     search_results = catalog_live.search(mock_search_parameters)
     assert isinstance(search_results, gpd.GeoDataFrame)
-    assert search_results.shape == (4, 10)
+    assert search_results.shape == (4, 9)
     assert list(search_results.columns) == [
-        "index",
         "geometry",
         "id",
         "acquisitionDate",
