@@ -9,8 +9,8 @@ import pytest
 from folium import Map
 
 # pylint: disable=unused-import
+from .context import Job, JobTask
 from .fixtures import auth_mock, auth_live, job_mock, job_live, jobtask_mock
-import up42  # pylint: disable=wrong-import-order
 
 
 def test_job_get_info(job_mock):
@@ -24,7 +24,7 @@ def test_job_get_info(job_mock):
         m.get(url=url_job_info, text='{"data": {"xyz":789}, "error":{}}')
 
         info = job_mock._get_info()
-    assert isinstance(job_mock, up42.Job)
+    assert isinstance(job_mock, Job)
     assert info["xyz"] == 789
     assert job_mock.info["xyz"] == 789
 
@@ -45,6 +45,31 @@ def test_get_status(job_mock, status):
     assert job_status == status
 
 
+# pylint: disable=unused-argument
+@pytest.mark.parametrize(
+    "status,expected",
+    [
+        ("NOT STARTED", False),
+        ("PENDING", False),
+        ("RUNNING", False),
+        ("FAILED", False),
+        ("SUCCEEDED", True),
+    ],
+)
+def test_is_succeeded(job_mock, status, expected):
+    del job_mock.info
+
+    with requests_mock.Mocker() as m:
+        url_job_info = (
+            f"{job_mock.auth._endpoint()}/projects/"
+            f"{job_mock.project_id}/jobs/{job_mock.job_id}"
+        )
+        m.get(url=url_job_info, json={"data": {"status": status}, "error": {}})
+
+        job_is_succeeded = job_mock.is_succeeded
+    assert job_is_succeeded == expected
+
+
 @pytest.mark.parametrize("status", ["SUCCEEDED"])
 def test_track_status_pass(job_mock, status):
     del job_mock.info
@@ -145,6 +170,7 @@ def test_get_logs(job_mock, jobtask_mock):
         )
         m.get(url_log, json="")
         assert job_mock.get_logs(as_return=True)[jobtask_mock.jobtask_id] == ""
+        assert not job_mock.get_logs()
 
 
 def test_get_jobtasks(job_mock, jobtask_mock):
@@ -155,7 +181,7 @@ def test_get_jobtasks(job_mock, jobtask_mock):
         )
         m.get(url=url_job_tasks, json={"data": [{"id": jobtask_mock.jobtask_id}]})
         job_tasks = job_mock.get_jobtasks()
-        assert isinstance(job_tasks[0], up42.JobTask)
+        assert isinstance(job_tasks[0], JobTask)
         assert job_tasks[0].jobtask_id == jobtask_mock.jobtask_id
 
 
@@ -198,9 +224,37 @@ def test_job_download_result(job_mock):
 
         with tempfile.TemporaryDirectory() as tempdir:
             out_files = job_mock.download_results(tempdir)
+            out_paths = [Path(p) for p in out_files]
+            for path in out_paths:
+                assert path.exists()
+            assert len(out_paths) == 2
+            assert out_paths[0].name == "data.json"
+            assert out_paths[1].parent.exists()
+            assert out_paths[1].parent.is_dir()
+
+
+def test_job_download_result_nounpacking(job_mock):
+    with requests_mock.Mocker() as m:
+        download_url = "http://up42.api.com/abcdef"
+        url_download_result = (
+            f"{job_mock.auth._endpoint()}/projects/"
+            f"{job_mock.project_id}/jobs/{job_mock.job_id}/downloads/results/"
+        )
+        m.get(url_download_result, json={"data": {"url": download_url}, "error": {}})
+
+        out_tgz = Path(__file__).resolve().parent / "mock_data/result_tif.tgz"
+        out_tgz_file = open(out_tgz, "rb")
+        m.get(
+            url=download_url,
+            content=out_tgz_file.read(),
+            headers={"x-goog-stored-content-length": "163"},
+        )
+
+        with tempfile.TemporaryDirectory() as tempdir:
+            out_files = job_mock.download_results(tempdir, unpacking=False)
             for file in out_files:
                 assert Path(file).exists()
-            assert len(out_files) == 2
+            assert len(out_files) == 1
 
 
 @pytest.mark.live
@@ -215,23 +269,40 @@ def test_job_download_result_live(job_live):
 @pytest.mark.live
 def test_job_download_result_no_tiff_live(auth_live):
     with tempfile.TemporaryDirectory() as tempdir:
-        job = up42.Job(
+        job = Job(
             auth=auth_live,
             project_id=auth_live.project_id,
             job_id=os.getenv("TEST_UP42_JOB_ID_NC_FILE"),
         )
         out_files = job.download_results(Path(tempdir))
         assert Path(out_files[0]).exists()
-        assert Path(out_files[0]).suffix == ".nc"
+        assert Path(out_files[1]).exists()
+        assert any(".nc" in s for s in out_files)
+        assert any("data.json" in s for s in out_files)
+        assert len(out_files) == 2
+
+
+@pytest.mark.live
+def test_job_download_result_dimap_live(auth_live):
+    with tempfile.TemporaryDirectory() as tempdir:
+        job = Job(
+            auth=auth_live,
+            project_id=auth_live.project_id,
+            job_id=os.getenv("TEST_UP42_JOB_ID_DIMAP_FILE"),
+        )
+        out_files = job.download_results(Path(tempdir))
+        print(out_files)
+        assert Path(out_files[0]).exists()
+        assert Path(out_files[20]).exists()
         assert Path(out_files[1]).exists()
         assert Path(out_files[1]).name == "data.json"
-        assert len(out_files) == 2
+        assert len(out_files) == 44
 
 
 @pytest.mark.skip
 @pytest.mark.live
 def test_job_download_result_live_2gb_big_exceeding_2min_gcs_treshold(auth_live):
-    job = up42.Job(
+    job = Job(
         auth=auth_live,
         project_id=auth_live.project_id,
         job_id="30f82b44-1505-4773-ab23-31fa61ba9b4c",
