@@ -7,14 +7,14 @@ from geojson import FeatureCollection
 from geopandas import GeoDataFrame
 
 # pylint: disable=wrong-import-position
-from .tools import Tools
-from .auth import Auth
-from .project import Project
-from .workflow import Workflow
-from .job import Job
-from .jobtask import JobTask
-from .catalog import Catalog
-from .utils import get_logger
+from up42.tools import Tools
+from up42.auth import Auth
+from up42.project import Project
+from up42.workflow import Workflow
+from up42.job import Job
+from up42.jobtask import JobTask
+from up42.catalog import Catalog
+from up42.utils import get_logger
 
 logger = get_logger(__name__, level=logging.INFO)
 
@@ -23,13 +23,18 @@ warnings.simplefilter(action="ignore", category=FutureWarning)
 _auth = None
 
 
-def authenticate(cfg_file=None, project_id=None, project_api_key=None, **kwargs):
+def authenticate(
+    cfg_file: Union[str, Path] = None,
+    project_id: str = None,
+    project_api_key: str = None,
+    **kwargs,
+):
     global _auth  # pylint: disable=global-statement
     _auth = Auth(
         cfg_file=cfg_file,
         project_id=project_id,
         project_api_key=project_api_key,
-        **kwargs
+        **kwargs,
     )
 
 
@@ -37,8 +42,8 @@ def initialize_project() -> "Project":
     """Directly returns the correct Project object (has to exist on UP42)."""
     if _auth is None:
         raise RuntimeError("Not authenticated, call up42.authenticate() first")
-    logger.info("Working on Project with project_id %s", _auth.project_id)
-    return Project(auth=_auth, project_id=_auth.project_id)
+    logger.info(f"Working on Project with project_id {_auth.project_id}")
+    return Project(auth=_auth, project_id=str(_auth.project_id))
 
 
 def initialize_catalog() -> "Catalog":
@@ -52,7 +57,9 @@ def initialize_workflow(workflow_id) -> "Workflow":
     """Directly returns a Workflow object (has to exist on UP42)."""
     if _auth is None:
         raise RuntimeError("Not authenticated, call up42.authenticate() first")
-    return Workflow(auth=_auth, workflow_id=workflow_id, project_id=_auth.project_id)
+    return Workflow(
+        auth=_auth, workflow_id=workflow_id, project_id=str(_auth.project_id)
+    )
 
 
 def initialize_job(job_id, order_ids: List[str] = None) -> "Job":
@@ -60,7 +67,7 @@ def initialize_job(job_id, order_ids: List[str] = None) -> "Job":
     if _auth is None:
         raise RuntimeError("Not authenticated, call up42.authenticate() first")
     return Job(
-        auth=_auth, job_id=job_id, project_id=_auth.project_id, order_ids=order_ids
+        auth=_auth, job_id=job_id, project_id=str(_auth.project_id), order_ids=order_ids
     )
 
 
@@ -69,7 +76,10 @@ def initialize_jobtask(jobtask_id, job_id) -> "JobTask":
     if _auth is None:
         raise RuntimeError("Not authenticated, call up42.authenticate() first")
     return JobTask(
-        auth=_auth, jobtask_id=jobtask_id, job_id=job_id, project_id=_auth.project_id
+        auth=_auth,
+        jobtask_id=jobtask_id,
+        job_id=job_id,
+        project_id=str(_auth.project_id),
     )
 
 
