@@ -4,13 +4,13 @@ from pathlib import Path
 from datetime import datetime, timedelta
 import click
 
-from .auth import Auth
-from .tools import Tools
-from .project import Project
-from .workflow import Workflow
-from .job import Job
-from .catalog import Catalog
-from .utils import get_logger
+from up42.auth import Auth
+from up42.tools import Tools
+from up42.project import Project
+from up42.workflow import Workflow
+from up42.job import Job
+from up42.catalog import Catalog
+from up42.utils import get_logger
 
 logger = get_logger(__name__)
 
@@ -19,9 +19,6 @@ CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"], show_default=True)
 # To activate bash autocompletion
 # eval "$(_UP42_COMPLETE=source_bash up42)"
 
-# For usage of fstrings
-# pylint: disable=logging-format-interpolation
-
 
 def pprint_json(obj, indent=2):
     return "\n" + json.dumps(obj, indent=indent, sort_keys=True)
