@@ -1,7 +1,7 @@
 import logging
 from pathlib import Path
 from time import sleep
-from typing import Dict, List, Union
+from typing import Dict, List, Union, Optional
 
 import folium
 from geopandas import GeoDataFrame
@@ -13,13 +13,14 @@ from shapely.geometry import box
 from rasterio.vrt import WarpedVRT
 import rasterio
 
-from .auth import Auth
-from .jobtask import JobTask
-from .tools import Tools
-from .utils import (
+from up42.auth import Auth
+from up42.jobtask import JobTask
+from up42.tools import Tools
+from up42.utils import (
     get_logger,
     folium_base_map,
     download_results_from_gcs,
+    download_results_from_gcs_without_unpacking,
 )
 
 try:
@@ -46,15 +47,14 @@ class Job(Tools):
         self.job_id = job_id
         self.quicklooks = None
         self.results = None
-        if order_ids is None:
-            self.order_ids = [""]
+        self.order_ids = order_ids
         if self.auth.get_info:
             self.info = self._get_info()
 
     def __repr__(self):
         return (
             f"Job(job_id={self.job_id}, project_id={self.project_id}, "
-            f"order_ids={str(self.order_ids)}, auth={self.auth}, info={self.info})"
+            f"order_ids={self.order_ids}, auth={self.auth}, info={self.info})"
         )
 
     def _get_info(self):
@@ -69,14 +69,19 @@ class Job(Tools):
         Gets the job status.
 
         Returns:
-            The job status.
+            The job status, one of "SUCCEEDED", "NOT STARTED", "PENDING", "RUNNING",
+            "CANCELLED", "CANCELLING", "FAILED", "ERROR"
         """
-        # logger.info("Getting job status: %s", self.job_id)
         info = self._get_info()
         status = info["status"]
-        logger.info("Job is %s", status)
+        logger.info(f"Job is {status}")
         return status
 
+    @property
+    def is_succeeded(self):
+        """Gets True if the job succeeded, False otherwise"""
+        return self.get_status() == "SUCCEEDED"
+
     def track_status(self, report_time: int = 30) -> str:
         """
         Continuously gets the job status until job has finished or failed.
@@ -88,8 +93,7 @@ class Job(Tools):
             report_time: The intervall (in seconds) when to query the job status.
         """
         logger.info(
-            "Tracking job status continuously, reporting every %s seconds...",
-            report_time,
+            f"Tracking job status continuously, reporting every {report_time} seconds...",
         )
         status = "NOT STARTED"
         time_asleep = 0
@@ -99,18 +103,19 @@ class Job(Tools):
             status = self.get_status()
             logger.setLevel(logging.INFO)
 
+            # TODO: Add statuses as constants (maybe objects?)
             if status in ["NOT STARTED", "PENDING", "RUNNING"]:
                 if time_asleep != 0 and time_asleep % report_time == 0:
-                    logger.info("Job is %s! - %s", status, self.job_id)
+                    logger.info(f"Job is {status}! - {self.job_id}")
             elif status in ["FAILED", "ERROR"]:
-                logger.info("Job is %s! - %s - Printing logs ...", status, self.job_id)
+                logger.info(f"Job is {status}! - {self.job_id} - Printing logs ...")
                 self.get_logs(as_print=True)
                 raise ValueError("Job has failed! See the above log.")
             elif status in ["CANCELLED", "CANCELLING"]:
-                logger.info("Job is %s! - %s", status, self.job_id)
+                logger.info(f"Job is {status}! - {self.job_id}")
                 raise ValueError("Job has been cancelled!")
             elif status == "SUCCEEDED":
-                logger.info("Job finished successfully! - %s", self.job_id)
+                logger.info(f"Job finished successfully! - {self.job_id}")
 
             sleep(5)
             time_asleep += 5
@@ -121,7 +126,7 @@ class Job(Tools):
         """Cancels a pending or running job."""
         url = f"{self.auth._endpoint()}/jobs/{self.job_id}/cancel/"
         self.auth._request(request_type="POST", url=url)
-        logger.info("Job canceled: %s", self.job_id)
+        logger.info(f"Job canceled: {self.job_id}")
 
     def download_quicklooks(
         self, output_directory: Union[str, Path, None] = None
@@ -157,7 +162,7 @@ class Job(Tools):
             f"/outputs/data-json/"
         )
         response_json = self.auth._request(request_type="GET", url=url)
-        logger.info("Retrieved %s features.", len(response_json["features"]))
+        logger.info(f"Retrieved {len(response_json['features'])} features.")
 
         if as_dataframe:
             # UP42 results are always in EPSG 4326
@@ -176,20 +181,20 @@ class Job(Tools):
         return download_url
 
     def download_results(
-        self, output_directory: Union[str, Path, None] = None,
+        self, output_directory: Union[str, Path, None] = None, unpacking: bool = True
     ) -> List[str]:
         """
-        Downloads and unpacks the job results.
+        Downloads the job results. Unpacking the final file will happen as default.
 
         Args:
             output_directory: The file output directory, defaults to the current working
                 directory.
+            unpacking: By default the final result which is in TAR archive format will be unpacked.
 
         Returns:
             List of the downloaded results' filepaths.
         """
-        # TODO: Overwrite argument
-        logger.info("Downloading results of job %s", self.job_id)
+        logger.info(f"Downloading results of job {self.job_id}")
 
         if output_directory is None:
             output_directory = (
@@ -198,12 +203,17 @@ class Job(Tools):
         else:
             output_directory = Path(output_directory)
         output_directory.mkdir(parents=True, exist_ok=True)
-        logger.info("Download directory: %s", str(output_directory))
+        logger.info(f"Download directory: {str(output_directory)}")
 
         download_url = self._get_download_url()
-        out_filepaths = download_results_from_gcs(
-            download_url=download_url, output_directory=output_directory,
-        )
+        if unpacking:
+            out_filepaths = download_results_from_gcs(
+                download_url=download_url, output_directory=output_directory,
+            )
+        else:
+            out_filepaths = download_results_from_gcs_without_unpacking(
+                download_url=download_url, output_directory=output_directory,
+            )
 
         self.results = out_filepaths
         return out_filepaths
@@ -211,22 +221,25 @@ class Job(Tools):
     def upload_results_to_bucket(
         self, gs_client, bucket, folder, extension: str = ".tgz", version: str = "v0"
     ) -> None:
-        """Uploads the results to a custom google cloud storage bucket."""
+        """
+        Uploads the results of a job directly to a custom google cloud storage bucket.
+        """
         download_url = self._get_download_url()
         r = requests.get(download_url)
 
-        if self.order_ids != [""]:
+        if self.order_ids is not None:
             blob = bucket.blob(
                 str(Path(version) / Path(folder) / Path(self.order_ids[0] + extension))
             )
             logger.info(
-                "Upload job %s results with order_ids to %s...", self.job_id, blob.name
+                f"Upload job {self.job_id} results with order_ids to "
+                f"{blob.name} ..."
             )
         else:
             blob = bucket.blob(
                 str(Path(version) / Path(folder) / Path(self.job_id + extension))
             )
-            logger.info("Upload job %s results to %s...", self.job_id, blob.name)
+            logger.info(f"Upload job {self.job_id} results to {blob.name} ...")
         blob.upload_from_string(
             data=r.content, content_type="application/octet-stream", client=gs_client,
         )
@@ -240,8 +253,6 @@ class Job(Tools):
             show_images: Shows images if True (default), only features if False.
             name_column: Name of the feature property that provides the Feature/Layer name.
         """
-        # TODO: Make generic with scene_id column integrated.
-        # TODO: Add way to also display data block image together with processing output vectors
         if self.results is None:
             raise ValueError(
                 "You first need to download the results via job.download_results()!"
@@ -338,7 +349,9 @@ class Job(Tools):
             )
             return m
 
-    def get_logs(self, as_print: bool = True, as_return: bool = False):
+    def get_logs(
+        self, as_print: bool = True, as_return: bool = False
+    ) -> Optional[Dict]:
         """
         Convenience function to print or return the logs of all job tasks.
 
@@ -349,14 +362,12 @@ class Job(Tools):
         Returns:
             The log strings (only if as_return was selected).
         """
+        job_logs = {}
+
         jobtasks: List[Dict] = self.get_jobtasks(return_json=True)  # type: ignore
         jobtasks_ids = [task["id"] for task in jobtasks]
 
-        logger.info(
-            "Getting logs for %s job tasks: %s", len(jobtasks_ids), jobtasks_ids
-        )
-        job_logs = {}
-
+        logger.info(f"Getting logs for {len(jobtasks_ids)} job tasks: {jobtasks_ids}")
         if as_print:
             print(
                 f"Printing logs of {len(jobtasks_ids)} JobTasks in Job with job_id "
@@ -376,8 +387,10 @@ class Job(Tools):
                 print("----------------------------------------------------------")
                 print(f"JobTask {idx+1} with jobtask_id {jobtask_id}:\n")
                 print(response_json)
-            if as_return:
-                return job_logs
+        if as_return:
+            return job_logs
+        else:
+            return None
 
     def get_jobtasks(
         self, return_json: bool = False
@@ -395,7 +408,7 @@ class Job(Tools):
             f"{self.auth._endpoint()}/projects/{self.project_id}/jobs/{self.job_id}"
             f"/tasks/"
         )
-        logger.info("Getting job tasks: %s", self.job_id)
+        logger.info(f"Getting job tasks: {self.job_id}")
         response_json = self.auth._request(request_type="GET", url=url)
         jobtasks_json: List[Dict] = response_json["data"]
 
