@@ -10,7 +10,7 @@ import pandas as pd
 import shapely
 import rasterio
 
-from .utils import get_logger, folium_base_map, DrawFoliumOverride, _plot_images
+from up42.utils import get_logger, folium_base_map, DrawFoliumOverride, _plot_images
 
 try:
     from IPython.display import display
@@ -73,9 +73,7 @@ class Tools:
             df = df.to_crs(epsg=4326)
         df.geometry = df.geometry.buffer(0)
         # TODO: Explode multipolygons (if neccessary as union in aoi anyway most often).
-
         # TODO: Have both bboxes for each feature and overall?
-
         if as_dataframe:
             return df
         else:
@@ -95,7 +93,7 @@ class Tools:
         Returns:
             Feature collection json with the selected aoi.
         """
-        logger.info("Getting small example aoi in %s.", location)
+        logger.info(f"Getting small example aoi in location '{location}'.")
         if location == "Berlin":
             example_aoi = self.read_vector_file(
                 f"{str(Path(__file__).resolve().parent)}/data/aoi_berlin.geojson"
@@ -106,7 +104,7 @@ class Tools:
             )
         else:
             raise ValueError(
-                "Please select one of 'Berlin' or 'Washington' as the " "location!"
+                "Please select one of 'Berlin' or 'Washington' as the location!"
             )
 
         if as_dataframe:
@@ -228,7 +226,7 @@ class Tools:
 
     def plot_results(
         self,
-        figsize: Tuple[int, int] = (8, 8),
+        figsize: Tuple[int, int] = (14, 8),
         filepaths: List[Union[str, Path]] = None,
         titles: List[str] = None,
     ) -> None:
@@ -245,6 +243,11 @@ class Tools:
             if self.results is None:
                 raise ValueError("You first need to download the results!")
             filepaths = self.results
+            # Unpack results path dict in case of jobcollection.
+            if isinstance(filepaths, dict):
+                filepaths = [
+                    item for sublist in list(filepaths.values()) for item in sublist  # type: ignore
+                ]
 
         plot_file_format = [".tif"]  # TODO: Add other fileformats.
         _plot_images(
