@@ -2,6 +2,7 @@ import copy
 import logging
 from typing import Dict, List, Union, Tuple
 from pathlib import Path
+import shutil
 import tempfile
 import tarfile
 import math
@@ -20,7 +21,8 @@ from tqdm import tqdm
 import matplotlib.pyplot as plt
 
 
-LOG_FORMAT = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
+# truncate log messages > 2000 characters (e.g. huge geometries)
+LOG_FORMAT = "%(asctime)s - %(name)s - %(levelname)s - %(message).2000s"
 
 
 def get_logger(name, level=logging.INFO):
@@ -66,29 +68,60 @@ def download_results_from_gcs(
                 if chunk:  # filter out keep-alive new chunks
                     dst_tgz.write(chunk)
         except requests.exceptions.HTTPError as err:
-            logger.debug("Connection error, please try again! %s", err)
+            logger.debug(f"Connection error, please try again! {err}")
             raise requests.exceptions.HTTPError(
                 f"Connection error, please try again! {err}"
             )
 
-    # Unpack and exclude data.json
-    out_filepaths: List[str] = []
     with tarfile.open(tgz_file) as tar:
-        members = tar.getmembers()
-        files = [f for f in members if f.isfile()]
-        for file in files:
-            f = tar.extractfile(file)
-            content = f.read()  # type: ignore
-            out_fp = output_directory / f"{Path(file.name).name}"
-            with open(out_fp, "wb") as dst:
-                dst.write(content)
-            out_filepaths.append(str(out_fp))
+        tar.extractall(path=output_directory)
+        output_folder_path = output_directory / "output"
+        for src_path in output_folder_path.glob("**/*"):
+            dst_path = output_directory / src_path.relative_to(output_folder_path)
+            shutil.move(str(src_path), str(dst_path))
+        out_filepaths = [str(x) for x in output_directory.glob("**/*") if x.is_file()]
 
     logger.info(
-        "Download successful of %s files to output_directory '%s': %s",
-        len(out_filepaths),
-        output_directory,
-        [Path(p).name for p in out_filepaths],
+        f"Download successful of {len(out_filepaths)} files to output_directory "
+        f"'{output_directory}': {[Path(p).name for p in out_filepaths]}"
+    )
+    return out_filepaths
+
+
+def download_results_from_gcs_without_unpacking(
+    download_url: str, output_directory: Union[str, Path]
+) -> List[str]:
+    """
+    General download function for results of job and jobtask from cloud storage
+    provider.
+
+    Args:
+        download_url: The signed gcs url to download.
+        output_directory: The file output directory, defaults to the current working
+            directory.
+    """
+    output_directory = Path(output_directory)
+
+    # Download
+    out_filepaths: List[str] = []
+    out_fp = Path().joinpath(output_directory, "output.tgz")
+    with open(out_fp, "wb") as dst:
+        try:
+            r = requests.get(download_url)
+            r.raise_for_status()
+            for chunk in tqdm(r.iter_content(chunk_size=1024)):
+                if chunk:  # filter out keep-alive new chunks
+                    dst.write(chunk)
+            out_filepaths.append(str(out_fp))
+        except requests.exceptions.HTTPError as err:
+            logger.debug(f"Connection error, please try again! {err}")
+            raise requests.exceptions.HTTPError(
+                f"Connection error, please try again! {err}"
+            )
+
+    logger.info(
+        f"Download successful of {len(out_filepaths)} files to output_directory"
+        f" '{output_directory}': {[Path(p).name for p in out_filepaths]}"
     )
     return out_filepaths
 
@@ -205,7 +238,7 @@ class DrawFoliumOverride(Draw):
 
 def _plot_images(
     plot_file_format: List[str],
-    figsize: Tuple[int, int] = (8, 8),
+    figsize: Tuple[int, int] = (14, 8),
     filepaths: List[Union[str, Path]] = None,
     titles: List[str] = None,
 ) -> None:
@@ -248,15 +281,21 @@ def _plot_images(
         axs = axs.ravel()
     else:
         axs = [axs]
+
     for idx, (fp, title) in enumerate(zip(imagepaths, titles)):
         with rasterio.open(fp) as src:
             img_array = src.read()[:3, :, :]
             # TODO: Handle more band configurations.
             # TODO: add histogram equalization?
             show(
-                img_array, transform=src.transform, title=title, ax=axs[idx],
+                img_array,
+                transform=src.transform,
+                title=title,
+                ax=axs[idx],
+                aspect="auto",
             )
         axs[idx].set_axis_off()
+    plt.axis("off")
     plt.tight_layout()
     plt.show()
 
@@ -301,6 +340,7 @@ def any_vector_to_fc(
             if vector["type"] == "FeatureCollection":
                 df = GeoDataFrame.from_features(vector, crs=4326)
             elif vector["type"] == "Feature":
+                # TODO: Handle point features?
                 df = GeoDataFrame.from_features(FeatureCollection([vector]), crs=4326)
             elif vector["type"] == "Polygon":  # Geojson geometry
                 df = GeoDataFrame.from_features(
@@ -394,9 +434,8 @@ def fc_to_query_geometry(
     # geometry via handle_multiple_features method.
     else:
         logger.info(
-            "The provided geometry contains multiple geometries, the %s feature is "
-            "taken instead.",
-            squash_multiple_features,
+            f"The provided geometry contains multiple geometries, "
+            f"the {squash_multiple_features} feature is taken instead."
         )
         if geometry_operation == "bbox":
             if squash_multiple_features == "union":
